import { Connection, PublicKey, Keypair, TransactionInstruction } from '@solana/web3.js';
import { Program, AnchorProvider, BN } from '@coral-xyz/anchor';
import { OmnipumpSolana } from './idls/omnipump_solana';
import { BondingCurveInfo, BoughtToken, CompleteEvent, ConfigInfo, CreateTokenEvent, OmniEventName, ReferralInfo, TradeEvent } from './states';
export declare class OmniPumpSDK {
    program: Program<OmnipumpSolana>;
    configInfo: ConfigInfo | undefined;
    connection: Connection;
    provider: AnchorProvider;
    constructor(connection: Connection, programId: PublicKey);
    init(): Promise<void>;
    getConfigInfo(): Promise<ConfigInfo>;
    getBondingCurveInfosFromMints(mints: PublicKey[]): Promise<(BondingCurveInfo | undefined)[]>;
    getBondingCurveInfos(bondingCurves: PublicKey[]): Promise<(BondingCurveInfo | undefined)[]>;
    getBoughtTokens(mints: PublicKey[], owner: PublicKey): Promise<(BoughtToken | undefined)[]>;
    getMultiBoughtTokens(boughtTokens: PublicKey[]): Promise<(BoughtToken | undefined)[]>;
    getReferralInfo(user: PublicKey): Promise<ReferralInfo | undefined>;
    prepareCreateIx(name: string, symbol: string, uri: string, tokenCreator: PublicKey, isPumpfunCreator?: boolean, buyAmount?: BN): Promise<{
        ix: TransactionInstruction;
        buyIx: TransactionInstruction | undefined;
        mint: Keypair;
    }>;
    prepareTradeIx(mint: PublicKey, user: PublicKey, isBuy: boolean, amount: BN, referrer: PublicKey | undefined, slipages: number[], isWithBuy?: boolean): Promise<TransactionInstruction>;
    prepareClaimIx(mint: PublicKey, user: PublicKey): Promise<TransactionInstruction>;
    onEvent(event: OmniEventName, callback: {
        (omniEvent: CreateTokenEvent | TradeEvent | CompleteEvent, signature: string): void;
    }): number;
    removeEventListener(id: number): void;
    getCreatedBondingCurves(minSlot?: number): Promise<{
        bondingCurves: PublicKey[];
        slot: number;
    }>;
    getCreatedBoughtTokens(minSlot?: number): Promise<{
        boughtTokens: PublicKey[];
        slot: number;
    }>;
    getCreatedReferralInfos(minSlot?: number): Promise<{
        referrals: PublicKey[];
        slot: number;
    }>;
}
