"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.OmniPumpSDK = void 0;
/* eslint-disable @typescript-eslint/no-unsafe-call */
/* eslint-disable @typescript-eslint/no-unsafe-member-access */
const web3_js_1 = require("@solana/web3.js");
const js_sha256_1 = require("js-sha256");
const anchor_1 = require("@coral-xyz/anchor");
const IDL = __importStar(require("./idls/omnipump_solana.json"));
const curve_1 = require("./curve");
const pda_1 = require("./pda");
const spl_token_1 = require("@solana/spl-token");
const constants_1 = require("./constants");
const bytes_1 = require("@coral-xyz/anchor/dist/cjs/utils/bytes");
const nodewallet_1 = __importDefault(require("@coral-xyz/anchor/dist/cjs/nodewallet"));
const METADATA_SEED = 'metadata';
class OmniPumpSDK {
    constructor(connection, programId) {
        this.provider = new anchor_1.AnchorProvider(connection, new nodewallet_1.default(web3_js_1.Keypair.generate()), {});
        this.connection = connection;
        const idl = Object.assign({}, IDL);
        // eslint-disable-next-line @typescript-eslint/no-unsafe-member-access
        idl.address = programId.toBase58(); // eslint-disable-line @typescript-eslint/no-explicit-any
        this.program = new anchor_1.Program(idl, this.provider);
    }
    init() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.getConfigInfo();
        });
    }
    getConfigInfo() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.configInfo) {
                return this.configInfo;
            }
            const configInfoState = yield this.program.account.configInfo.fetch((0, pda_1.getConfigInfoPDA)(this.program.programId), 'processed');
            return {
                owner: configInfoState.owner,
                treasury: configInfoState.treasury,
                withdrawAuthority: configInfoState.withdrawAuthority,
                bump: configInfoState.bump,
                operator: configInfoState.operator,
            };
        });
    }
    getBondingCurveInfosFromMints(mints) {
        return __awaiter(this, void 0, void 0, function* () {
            const bondingCurveInfos = yield this.program.account.bondingCurve.fetchMultiple(mints.map(mint => (0, pda_1.getBondingCurvePDA)(this.program.programId, mint)), 'processed');
            return bondingCurveInfos.map(bondingCurveInfo => {
                if (!bondingCurveInfo) {
                    return undefined;
                }
                const totalTokenForSell = new anchor_1.BN(constants_1.INITIAL_VIRTUAL_TOKEN_RESERVE).sub(new anchor_1.BN(constants_1.VIRTUAL_TOKEN_RESERVE_WHEN_ADD_LIQUIDITY));
                const soldTokenAmount = new anchor_1.BN(constants_1.INITIAL_VIRTUAL_TOKEN_RESERVE).sub(bondingCurveInfo.virtualTokenReserve);
                return {
                    tokenCreator: bondingCurveInfo.tokenCreator,
                    bump: bondingCurveInfo.bump,
                    complete: bondingCurveInfo.complete,
                    liquidityAdded: bondingCurveInfo.liquidityAdded,
                    syncedSol: bondingCurveInfo.syncedSol,
                    mint: bondingCurveInfo.mint,
                    virtualSolReserve: bondingCurveInfo.virtualSolReserve,
                    virtualTokenReserve: bondingCurveInfo.virtualTokenReserve,
                    tokenDebt: bondingCurveInfo.tokenDebt,
                    progress: soldTokenAmount
                        .mul(new anchor_1.BN(100))
                        .div(totalTokenForSell)
                        .toNumber(),
                    isPumpfunCreator: bondingCurveInfo.pumpfunCreator,
                };
            });
        });
    }
    getBondingCurveInfos(bondingCurves) {
        return __awaiter(this, void 0, void 0, function* () {
            const bondingCurveInfos = yield this.program.account.bondingCurve.fetchMultiple(bondingCurves, 'processed');
            return bondingCurveInfos.map(bondingCurveInfo => {
                if (!bondingCurveInfo) {
                    return undefined;
                }
                const totalTokenForSell = new anchor_1.BN(constants_1.INITIAL_VIRTUAL_TOKEN_RESERVE).sub(new anchor_1.BN(constants_1.VIRTUAL_TOKEN_RESERVE_WHEN_ADD_LIQUIDITY));
                const soldTokenAmount = new anchor_1.BN(constants_1.INITIAL_VIRTUAL_TOKEN_RESERVE).sub(bondingCurveInfo.virtualTokenReserve);
                return {
                    tokenCreator: bondingCurveInfo.tokenCreator,
                    bump: bondingCurveInfo.bump,
                    complete: bondingCurveInfo.complete,
                    liquidityAdded: bondingCurveInfo.liquidityAdded,
                    syncedSol: bondingCurveInfo.syncedSol,
                    mint: bondingCurveInfo.mint,
                    virtualSolReserve: bondingCurveInfo.virtualSolReserve,
                    virtualTokenReserve: bondingCurveInfo.virtualTokenReserve,
                    tokenDebt: bondingCurveInfo.tokenDebt,
                    progress: soldTokenAmount
                        .mul(new anchor_1.BN(100))
                        .div(totalTokenForSell)
                        .toNumber(),
                    isPumpfunCreator: bondingCurveInfo.pumpfunCreator,
                };
            });
        });
    }
    getBoughtTokens(mints, owner) {
        return __awaiter(this, void 0, void 0, function* () {
            const boughtTokenInfos = yield this.program.account.boughtToken.fetchMultiple(mints.map(mint => (0, pda_1.getBoughtTokenPDA)(this.program.programId, mint, owner)), 'processed');
            return boughtTokenInfos.map(boughtTokenInfo => {
                if (!boughtTokenInfo) {
                    return undefined;
                }
                return {
                    owner: boughtTokenInfo.owner,
                    mint: boughtTokenInfo.mint,
                    bump: boughtTokenInfo.bump,
                    balance: boughtTokenInfo.balance,
                };
            });
        });
    }
    getMultiBoughtTokens(boughtTokens) {
        return __awaiter(this, void 0, void 0, function* () {
            const boughtTokenInfos = yield this.program.account.boughtToken.fetchMultiple(boughtTokens, 'processed');
            return boughtTokenInfos.map(boughtTokenInfo => {
                if (!boughtTokenInfo) {
                    return undefined;
                }
                return {
                    owner: boughtTokenInfo.owner,
                    mint: boughtTokenInfo.mint,
                    bump: boughtTokenInfo.bump,
                    balance: boughtTokenInfo.balance,
                };
            });
        });
    }
    getReferralInfo(user) {
        return __awaiter(this, void 0, void 0, function* () {
            const referralInfo = yield this.program.account.referralInfo.fetchNullable((0, pda_1.getReferralInfoPDA)(this.program.programId, user), 'processed');
            if (!referralInfo) {
                return undefined;
            }
            return {
                referrer: referralInfo.referrer,
                user: referralInfo.user,
                bump: referralInfo.bump,
                initialized: referralInfo.initialized,
            };
        });
    }
    prepareCreateIx(name_1, symbol_1, uri_1, tokenCreator_1) {
        return __awaiter(this, arguments, void 0, function* (name, symbol, uri, tokenCreator, isPumpfunCreator = false, buyAmount) {
            const mint = web3_js_1.Keypair.generate();
            const TOKEN_METADATA_PROGRAM_ID = new web3_js_1.PublicKey('metaqbxxUerdq28cj1RbAWkYQm3ybzjb6a8bt518x1s');
            const [metadataAddress] = web3_js_1.PublicKey.findProgramAddressSync([
                Buffer.from(METADATA_SEED),
                TOKEN_METADATA_PROGRAM_ID.toBuffer(),
                mint.publicKey.toBuffer(),
            ], TOKEN_METADATA_PROGRAM_ID);
            const bondingCurve = (0, pda_1.getBondingCurvePDA)(this.program.programId, mint.publicKey);
            const bondingCurveAta = (0, spl_token_1.getAssociatedTokenAddressSync)(mint.publicKey, bondingCurve, true, spl_token_1.TOKEN_2022_PROGRAM_ID);
            const configInfoState = yield this.getConfigInfo();
            const accounts = {
                operator: isPumpfunCreator ? configInfoState.operator : null,
                payer: tokenCreator,
                metadata: metadataAddress,
                mint: mint.publicKey,
                bondingCurve,
                bondingCurveAta,
                treasury: configInfoState.treasury,
                configInfo: (0, pda_1.getConfigInfoPDA)(this.program.programId),
                tokenProgram2022: spl_token_1.TOKEN_2022_PROGRAM_ID,
            };
            return {
                ix: yield this.program.methods
                    .create(name, symbol, uri)
                    .accounts(accounts)
                    .signers([mint])
                    .instruction(),
                buyIx: buyAmount
                    ? yield this.prepareTradeIx(mint.publicKey, tokenCreator, true, buyAmount, undefined, [10, 100], true)
                    : undefined,
                mint,
            };
        });
    }
    prepareTradeIx(mint_1, user_1, isBuy_1, amount_1, referrer_1, slipages_1) {
        return __awaiter(this, arguments, void 0, function* (mint, user, isBuy, amount, referrer, slipages, isWithBuy = false) {
            const bondingCurve = (0, pda_1.getBondingCurvePDA)(this.program.programId, mint);
            const referralInfo = (0, pda_1.getReferralInfoPDA)(this.program.programId, user);
            const referralInfoStateBefore = yield this.program.account.referralInfo.fetchNullable(referralInfo, 'processed');
            if (referralInfoStateBefore && referralInfoStateBefore.referrer) {
                referrer = referralInfoStateBefore.referrer;
            }
            let referralInfoForDl1 = null;
            let referrerDl1 = null;
            if (referrer &&
                referralInfoStateBefore &&
                referralInfoStateBefore.referrer &&
                referralInfoStateBefore.referrer.toBase58() == referrer.toBase58()) {
                referralInfoForDl1 = (0, pda_1.getReferralInfoPDA)(this.program.programId, referrer);
                const referralInfoForDl1State = yield this.program.account.referralInfo.fetchNullable(referralInfoForDl1, 'processed');
                referrerDl1 =
                    referralInfoForDl1State && referralInfoForDl1State.referrer
                        ? referralInfoForDl1State.referrer
                        : null;
            }
            let referralInfoForDl2 = null;
            let referrerDl2 = null;
            const bondingCurveState = !isWithBuy
                ? yield this.program.account.bondingCurve.fetch(bondingCurve, 'processed')
                : {
                    virtualSolReserve: new anchor_1.BN(constants_1.INITIAL_VIRTUAL_SOL_RESERVE),
                    virtualTokenReserve: new anchor_1.BN(constants_1.INITIAL_VIRTUAL_TOKEN_RESERVE),
                    tokenCreator: user,
                };
            if (referrerDl1) {
                referralInfoForDl2 = (0, pda_1.getReferralInfoPDA)(this.program.programId, referrerDl1);
                const referralInfoForDl2State = yield this.program.account.referralInfo.fetchNullable(referralInfoForDl2, 'processed');
                referrerDl2 =
                    referralInfoForDl2State && referralInfoForDl2State.referrer
                        ? referralInfoForDl2State.referrer
                        : null;
            }
            if (isBuy) {
                const { amountOut } = (0, curve_1.getOutputTokenAmount)(amount, bondingCurveState.virtualSolReserve, bondingCurveState.virtualTokenReserve);
                const minAmountOut = amountOut
                    .mul(new anchor_1.BN(slipages[1] - slipages[0]))
                    .div(new anchor_1.BN(slipages[1]));
                const ix = yield this.program.methods
                    .buy(amount, minAmountOut)
                    .accounts({
                    payer: user,
                    mint: mint,
                    bondingCurve,
                    referralInfo,
                    referrer: referrer ? referrer : null,
                    referralInfoForDl1: referrerDl1 ? referralInfoForDl1 : null,
                    referralInfoForDl2: referrerDl2 ? referralInfoForDl2 : null,
                    referrerDl1: referrerDl1,
                    referrerDl2: referrerDl2,
                    tokenCreator: bondingCurveState.tokenCreator,
                    treasury: (yield this.getConfigInfo()).treasury,
                    configInfo: (0, pda_1.getConfigInfoPDA)(this.program.programId),
                    tokenProgram2022: spl_token_1.TOKEN_2022_PROGRAM_ID,
                })
                    .instruction();
                return ix;
            }
            const { outputSol } = (0, curve_1.getOutputSolAmount)(amount, bondingCurveState.virtualSolReserve, bondingCurveState.virtualTokenReserve);
            const minAmountOut = outputSol
                .mul(new anchor_1.BN(slipages[1] - slipages[0]))
                .div(new anchor_1.BN(slipages[1]));
            const ix = yield this.program.methods
                .sell(amount, minAmountOut)
                .accounts({
                payer: user,
                mint: mint,
                bondingCurve,
                referralInfo,
                referrer: referrer ? referrer : null,
                referralInfoForDl1: referrerDl1 ? referralInfoForDl1 : null,
                referralInfoForDl2: referrerDl2 ? referralInfoForDl2 : null,
                referrerDl1: referrerDl1,
                referrerDl2: referrerDl2,
                tokenCreator: bondingCurveState.tokenCreator,
                treasury: (yield this.getConfigInfo()).treasury,
                configInfo: (0, pda_1.getConfigInfoPDA)(this.program.programId),
                tokenProgram2022: spl_token_1.TOKEN_2022_PROGRAM_ID,
            })
                .instruction();
            return ix;
        });
    }
    prepareClaimIx(mint, user) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.program.methods
                .claim()
                .accounts({
                payer: user,
                mint,
                bondingCurve: (0, pda_1.getBondingCurvePDA)(this.program.programId, mint),
                tokenProgram2022: spl_token_1.TOKEN_2022_PROGRAM_ID,
            })
                .instruction();
        });
    }
    onEvent(event, callback) {
        return this.program.addEventListener(event, (eventDetails, _slot, signature) => {
            if (event == 'createTokenEvent') {
                callback({
                    eventName: event,
                    name: eventDetails.name,
                    symbol: eventDetails.symbol,
                    uri: eventDetails.uri,
                    mint: eventDetails.mint,
                    tokenCreator: eventDetails.tokenCreator,
                    timestamp: eventDetails.timestamp.toNumber(),
                }, signature);
            }
            else if (event == 'tradeEvent') {
                callback({
                    eventName: event,
                    solAmount: eventDetails.solAmount.toNumber(),
                    tokenAmount: eventDetails.tokenAmount,
                    mint: eventDetails.mint,
                    virtualSolReserve: eventDetails.virtualSolReserve,
                    virtualTokenReserve: eventDetails.virtualTokenReserve,
                    trader: eventDetails.trader,
                    isBuy: eventDetails.isBuy,
                    balance: eventDetails.balance,
                    timestamp: eventDetails.timestamp.toNumber(),
                }, signature);
            }
            else {
                callback({
                    eventName: event,
                    mint: eventDetails.mint,
                    timestamp: eventDetails.timestamp.toNumber(),
                }, signature);
            }
        });
    }
    removeEventListener(id) {
        this.program
            .removeEventListener(id)
            .then(() => {
            // do nothing
        })
            .catch(() => {
            // do nothing
        });
    }
    getCreatedBondingCurves(minSlot) {
        return __awaiter(this, void 0, void 0, function* () {
            const discriminator = Buffer.from(js_sha256_1.sha256.digest('account:BondingCurve')).slice(0, 8);
            const currentSlot = yield this.connection.getBlockHeight('processed');
            const bondingCurves = yield this.connection.getProgramAccounts(this.program.programId, {
                dataSlice: { offset: 0, length: 0 }, // Fetch without any data.
                filters: [
                    {
                        memcmp: {
                            offset: 0,
                            bytes: bytes_1.bs58.encode(discriminator),
                        },
                    },
                ],
                minContextSlot: minSlot,
                commitment: 'processed',
            });
            return {
                bondingCurves: bondingCurves.map(b => b.pubkey),
                slot: currentSlot,
            };
        });
    }
    getCreatedBoughtTokens(minSlot) {
        return __awaiter(this, void 0, void 0, function* () {
            const discriminator = Buffer.from(js_sha256_1.sha256.digest('account:BoughtToken')).slice(0, 8);
            const currentSlot = yield this.connection.getBlockHeight('processed');
            const boughtTokens = yield this.connection.getProgramAccounts(this.program.programId, {
                dataSlice: { offset: 0, length: 0 }, // Fetch without any data.
                filters: [
                    {
                        memcmp: {
                            offset: 0,
                            bytes: bytes_1.bs58.encode(discriminator),
                        },
                    },
                ],
                minContextSlot: minSlot,
                commitment: 'processed',
            });
            return {
                boughtTokens: boughtTokens.map(b => b.pubkey),
                slot: currentSlot,
            };
        });
    }
    getCreatedReferralInfos(minSlot) {
        return __awaiter(this, void 0, void 0, function* () {
            const discriminator = Buffer.from(js_sha256_1.sha256.digest('account:ReferralInfo')).slice(0, 8);
            const currentSlot = yield this.connection.getBlockHeight('processed');
            const referrals = yield this.connection.getProgramAccounts(this.program.programId, {
                dataSlice: { offset: 0, length: 0 }, // Fetch without any data.
                filters: [
                    {
                        memcmp: {
                            offset: 0,
                            bytes: bytes_1.bs58.encode(discriminator),
                        },
                    },
                ],
                minContextSlot: minSlot,
                commitment: 'processed',
            });
            return {
                referrals: referrals.map(b => b.pubkey),
                slot: currentSlot,
            };
        });
    }
}
exports.OmniPumpSDK = OmniPumpSDK;
